@@ -41,6 +41,7 @@
 #include "fillmac.h"
 #include "garp.h"
 #include "spawn.h"
+#include "log.h"
 #include "carp_p.h"
 
 #ifdef WITH_DMALLOC
@@ -54,20 +55,20 @@ static void carp_set_state(struct carp_softc *sc, int state)
     }
     switch (state) {
     case INIT:
-        puts("Switching to state : INIT");
+        logfile(LOG_INFO, _("Switching to state: INIT"));
         break;
     case BACKUP:
-        puts("Switching to state : BACKUP");
+        logfile(LOG_WARNING, _("Switching to state: BACKUP"));
         (void) spawn_handler(dev_desc_fd, downscript);
         gratuitous_arp(dev_desc_fd, 0);
         break;
     case MASTER:
-        puts("Switching to state : MASTER");
+        logfile(LOG_WARNING, _("Switching to state: MASTER"));
         (void) spawn_handler(dev_desc_fd, upscript);
         gratuitous_arp(dev_desc_fd, 1);
         break;
     default:
-        fprintf(stderr, "Unknown state : [%d]\n", (int) state);
+        logfile(LOG_ERR, _("Unknown state: [%d]"), (int) state);
         abort();
     }
     sc->sc_state = state;
@@ -102,18 +103,16 @@ void carp_hmac_prepare(struct carp_softc *sc)
     }
 }
 
-static unsigned short cksum(const void * const buf_, size_t len)
+static unsigned short cksum(const void * const buf_, const size_t len)
 {
     const unsigned char *buf = (unsigned char *) buf_;
-    unsigned long sum = 0U;
+    unsigned long sum = 0UL;
+    size_t evenlen = len & ~ (size_t) 1U;
     size_t i = (size_t) 0U;
     
     if (len <= (size_t) 0U) {
         return 0U;
     }
-    if ((len & 1) != 0) {
-        return 42U; /* XXX - TODO */
-    }
     do {
         sum += (buf[i] << 8) | buf[i + 1];
         if (sum > 0xffff) {
@@ -121,8 +120,14 @@ static unsigned short cksum(const void * const buf_, size_t len)
             sum++;
         }
         i += 2;
-    } while (i < len);
-    
+    } while (i < evenlen);
+    if (i != evenlen) {
+        sum += buf[i] << 8;
+        if (sum > 0xffff) {
+            sum &= 0xffff;
+            sum++;
+        }
+    }    
     return (unsigned short) ~sum;
 }
 
@@ -170,10 +175,8 @@ static void carp_send_ad(struct carp_softc *sc)
     struct carp_header ch;
     struct ether_header eh;
     struct timeval tv;
-    struct carp_header *ch_ptr;
-    struct ether_header *eh_ptr;
     struct ip ip;
-    struct ip *ip_ptr;
+    unsigned char *ip_ptr;
     unsigned char *pkt;
     unsigned short sum;    
     size_t ip_len;
@@ -182,7 +185,7 @@ static void carp_send_ad(struct carp_softc *sc)
     int advskew;
 
 #ifdef DEBUG
-    puts("-> carp_send_ad()");
+    logfile(LOG_DEBUG, "-> carp_send_ad()");
 #endif
     advbase = sc->sc_advbase;
     if (carp_suppress_preempt == 0 ||
@@ -206,7 +209,7 @@ static void carp_send_ad(struct carp_softc *sc)
     ip_len = sizeof ip + sizeof ch;
     eth_len = ip_len + sizeof eh;
     if ((pkt = ALLOCA(eth_len)) == NULL) {
-        perror("Out of memory to create packet");
+        logfile(LOG_ERR, _("Out of memory to create packet"));
         sc->sc_ad_tmo.tv_sec = now.tv_sec + tv.tv_sec;
         sc->sc_ad_tmo.tv_usec = now.tv_usec + tv.tv_usec;            
         return;
@@ -225,46 +228,53 @@ static void carp_send_ad(struct carp_softc *sc)
     
     ip.ip_dst.s_addr = INADDR_CARP_GROUP;
     
+    carp_prepare_ad(&ch, sc);       
+    
+    ch.carp_cksum = 0;
+    sum = cksum(&ch, sizeof ch);
+    ch.carp_cksum = htons(sum);    
+    
+    eh.ether_shost[0] = 0x00;
+    eh.ether_shost[1] = 0x00;
+    eh.ether_shost[2] = 0x5e;
+    eh.ether_shost[3] = 0x00;
+    eh.ether_shost[4] = 0x00;
+    eh.ether_shost[5] = vhid;
+    
+    eh.ether_dhost[0] = 0xff;
+    eh.ether_dhost[1] = 0xff;
+    eh.ether_dhost[2] = 0xff;
+    eh.ether_dhost[3] = 0xff;
+    eh.ether_dhost[4] = 0xff;
+    eh.ether_dhost[5] = 0xff;        
+    
+    eh.ether_type = htons(ETHERTYPE_IP);    
+    
     memcpy(pkt, &eh, sizeof eh);
     memcpy(pkt + sizeof eh, &ip, sizeof ip);
     memcpy(pkt + sizeof ip + sizeof eh, &ch, sizeof ch);
-    eh_ptr = (struct ether_header *) pkt;
-    ip_ptr = (struct ip *) (pkt + sizeof eh);        
-    ch_ptr = (struct carp_header *) (pkt + sizeof eh + sizeof ip);
-    
-    carp_prepare_ad(ch_ptr, sc);       
-    
-    ch_ptr->carp_cksum = 0;
-    sum = cksum(ch_ptr, sizeof ch);
-    ch_ptr->carp_cksum = htons(sum);
+
+    ip_ptr = pkt + sizeof eh;
     sum = cksum(ip_ptr, ip_len);
-    ip_ptr->ip_sum = htons(sum);
-    
-    eh_ptr->ether_shost[0] = 0x00;
-    eh_ptr->ether_shost[1] = 0x00;
-    eh_ptr->ether_shost[2] = 0x5e;
-    eh_ptr->ether_shost[3] = 0x00;
-    eh_ptr->ether_shost[4] = 0x00;
-    eh_ptr->ether_shost[5] = vhid;
-    
-    eh_ptr->ether_dhost[0] = 0xff;
-    eh_ptr->ether_dhost[1] = 0xff;
-    eh_ptr->ether_dhost[2] = 0xff;
-    eh_ptr->ether_dhost[3] = 0xff;
-    eh_ptr->ether_dhost[4] = 0xff;
-    eh_ptr->ether_dhost[5] = 0xff;        
-    
-    eh_ptr->ether_type = htons(ETHERTYPE_IP);
-    
+    ip_ptr[offsetof(struct ip, ip_sum)] = (sum >> 8) & 0xff;
+    ip_ptr[offsetof(struct ip, ip_sum) + 1] = sum & 0xff;
+        
     if (write(dev_desc_fd, pkt, eth_len) < 0) {
 #ifdef DEBUG
-        perror("write");
+        logfile(LOG_WARNING, _("write() has failed"));
 #endif
         if (sc->sc_sendad_errors < INT_MAX) {
             sc->sc_sendad_errors++;
         }
         if (sc->sc_sendad_errors == CARP_SENDAD_MAX_ERRORS) {
+#ifdef DEBUG
+            logfile(LOG_ERR, _("write() error #%d/%d"),
+                    carp_suppress_preempt, CARP_SENDAD_MAX_ERRORS);
+#endif            
             carp_suppress_preempt++;
+            if (carp_suppress_preempt == 1) {
+                carp_send_ad_all(sc);
+            }
         }
         sc->sc_sendad_success = 0;
     } else {
@@ -279,7 +289,7 @@ static void carp_send_ad(struct carp_softc *sc)
     }
     
 #ifdef DEBUG
-    puts("* advertisement injected *");
+    logfile(LOG_DEBUG, _("* advertisement injected *"));
 #endif
     
     ALLOCA_FREE(pkt);
@@ -290,12 +300,16 @@ static void carp_send_ad(struct carp_softc *sc)
     }
 }
 
+static void carp_send_ad_all(struct carp_softc *sc) {
+    return carp_send_ad(sc);
+}
+
 static void carp_setrun(struct carp_softc *sc, sa_family_t af)
 {
     struct timeval tv;    
     
 #ifdef DEBUG
-    puts("carp_setrun()");
+    logfile(LOG_DEBUG, "carp_setrun()");
 #endif
     switch (sc->sc_state) {
     case INIT:
@@ -308,6 +322,7 @@ static void carp_setrun(struct carp_softc *sc, sa_family_t af)
             carp_set_state(sc, MASTER);
         } else {
             carp_set_state(sc, BACKUP);
+            carp_setrun(sc, 0);
         }
         break;
     case BACKUP:
@@ -326,14 +341,12 @@ static void carp_setrun(struct carp_softc *sc, sa_family_t af)
             break;
 #endif /* INET6 */
         default:
-            if (sc->sc_md_tmo.tv_sec != 0) {
-                sc->sc_md_tmo.tv_sec = now.tv_sec + tv.tv_sec;
-                sc->sc_md_tmo.tv_usec = now.tv_usec + tv.tv_usec;
-            }
-            if (sc->sc_md6_tmo.tv_sec != 0) {
-                sc->sc_md6_tmo.tv_sec = now.tv_sec + tv.tv_sec;
-                sc->sc_md6_tmo.tv_usec = now.tv_usec + tv.tv_usec;
-            }
+            sc->sc_md_tmo.tv_sec = now.tv_sec + tv.tv_sec;
+            sc->sc_md_tmo.tv_usec = now.tv_usec + tv.tv_usec;
+#ifdef INET6
+            sc->sc_md6_tmo.tv_sec = now.tv_sec + tv.tv_sec;
+            sc->sc_md6_tmo.tv_usec = now.tv_usec + tv.tv_usec;
+#endif
             break;
         }        
         break;
@@ -350,12 +363,12 @@ static void carp_setrun(struct carp_softc *sc, sa_family_t af)
 static void carp_master_down(struct carp_softc *sc)
 {
 #ifdef DEBUG
-    puts("carp_master_down()");
+    logfile(LOG_DEBUG, "carp_master_down()");
 #endif
     switch (sc->sc_state) {
     case INIT:
 #ifdef DEBUG
-        fprintf(stderr, "master_down event in INIT state\n");
+        logfile(LOG_DEBUG, _("master_down event in INIT state"));
 #endif
         break;
     case MASTER:
@@ -385,27 +398,27 @@ static void packethandler(unsigned char *dummy,
     unsigned int ip_len;
             
     (void) dummy;
-    if (header->caplen < (sizeof etherhead + sizeof iphead)) {
+    if (header->caplen <= (sizeof etherhead + sizeof iphead)) {
         return;
     }    
     memcpy(&etherhead, sp, sizeof etherhead);
 #ifdef DEBUG
-    printf("Ethernet "
-           "[%02x:%02x:%02x:%02x:%02x:%02x]->[%02x:%02x:%02x:%02x:%02x:%02x] "
-           "type [%04x]\n",
-           (unsigned int) etherhead.ether_shost[0],
-           (unsigned int) etherhead.ether_shost[1],
-           (unsigned int) etherhead.ether_shost[2],
-           (unsigned int) etherhead.ether_shost[3],
-           (unsigned int) etherhead.ether_shost[4],
-           (unsigned int) etherhead.ether_shost[5],
-           (unsigned int) etherhead.ether_dhost[0],
-           (unsigned int) etherhead.ether_dhost[1],
-           (unsigned int) etherhead.ether_dhost[2],
-           (unsigned int) etherhead.ether_dhost[3],
-           (unsigned int) etherhead.ether_dhost[4],
-           (unsigned int) etherhead.ether_dhost[5],
-           (unsigned int) ntohs(etherhead.ether_type));
+    logfile(LOG_DEBUG, "Ethernet "
+             "[%02x:%02x:%02x:%02x:%02x:%02x]->[%02x:%02x:%02x:%02x:%02x:%02x] "
+             "type [%04x]",
+            (unsigned int) etherhead.ether_shost[0],
+            (unsigned int) etherhead.ether_shost[1],
+            (unsigned int) etherhead.ether_shost[2],
+            (unsigned int) etherhead.ether_shost[3],
+            (unsigned int) etherhead.ether_shost[4],
+            (unsigned int) etherhead.ether_shost[5],
+            (unsigned int) etherhead.ether_dhost[0],
+            (unsigned int) etherhead.ether_dhost[1],
+            (unsigned int) etherhead.ether_dhost[2],
+            (unsigned int) etherhead.ether_dhost[3],
+            (unsigned int) etherhead.ether_dhost[4],
+            (unsigned int) etherhead.ether_dhost[5],
+            (unsigned int) ntohs(etherhead.ether_type));
 #endif
     sp += sizeof etherhead;
     caplen = header->caplen - sizeof etherhead;
@@ -422,17 +435,18 @@ static void packethandler(unsigned char *dummy,
         struct timeval ch_tv;        
 
 #ifdef DEBUG
-        printf("\n---------------------------\n\n"
-               "carp [%d.%d.%d.%d] -> [%d.%d.%d.%d]\n",
-               source >> 24 & 0xff, source >> 16 & 0xff,
-               source >> 8 & 0xff, source & 0xff,
-               dest >> 24 & 0xff, dest >> 16 & 0xff,
-               dest >> 8 & 0xff, dest & 0xff);
+        logfile(LOG_DEBUG,
+                "\n---------------------------\n\n"
+                "carp [%d.%d.%d.%d] -> [%d.%d.%d.%d]",
+                source >> 24 & 0xff, source >> 16 & 0xff,
+                source >> 8 & 0xff, source & 0xff,
+                dest >> 24 & 0xff, dest >> 16 & 0xff,
+                dest >> 8 & 0xff, dest & 0xff);
 #endif
-        if (caplen != ip_len + sizeof ch) {
+        if (caplen < ip_len + sizeof ch) {
 #ifdef DEBUG
-            fprintf(stderr,
-                    "Bogus size : caplen=[%u], ip_len=[%u] ch_len=[%u]\n",
+            logfile(LOG_DEBUG,
+                    "Bogus size: caplen=[%u], ip_len=[%u] ch_len=[%u]",
                     (unsigned int) caplen, (unsigned int) ip_len,
                     (unsigned int) sizeof ch);
 #endif
@@ -440,37 +454,39 @@ static void packethandler(unsigned char *dummy,
         }
         memcpy(&ch, sp + ip_len, sizeof ch);
 #ifdef EXTRA_DEBUG
-        printf("Capture len : [%u] carp_header : [%u] ip_header : [%u]\n",
-               (unsigned int) caplen, (unsigned int) sizeof ch,
-               (unsigned int) ip_len);
-        printf("CARP type : [%u] version : [%u]\n",
-               (unsigned) ch.carp_type, (unsigned) ch.carp_version);
-        printf("CARP vhid : [%u] advskew : [%u]\n",
-               (unsigned) ch.carp_vhid, (unsigned) ch.carp_advskew);
-        printf("CARP advbase : [%u] cksum : [%u]\n",
-               (unsigned) ch.carp_advbase, (unsigned) ch.carp_cksum);
-        printf("CARP counter : [%lu][%lu]\n",
-               (unsigned long) ch.carp_counter[0],
-               (unsigned long) ch.carp_counter[1]);
+        logfile(LOG_DEBUG,
+                "Capture len: [%u] carp_header: [%u] ip_header: [%u]",
+                (unsigned int) caplen, (unsigned int) sizeof ch,
+                (unsigned int) ip_len);
+        logfile(LOG_DEBUG, "CARP type: [%u] version: [%u]",
+                (unsigned) ch.carp_type, (unsigned) ch.carp_version);
+        logfile(LOG_DEBUG, "CARP vhid: [%u] advskew: [%u]",
+                (unsigned) ch.carp_vhid, (unsigned) ch.carp_advskew);
+        logfile(LOG_DEBUG, "CARP advbase: [%u] cksum: [%u]",
+                (unsigned) ch.carp_advbase, (unsigned) ch.carp_cksum);
+        logfile(LOG_DEBUG, "CARP counter: [%lu][%lu]",
+                (unsigned long) ch.carp_counter[0],
+                (unsigned long) ch.carp_counter[1]);
 #endif
         if (iphead.ip_ttl != CARP_DFLTTL) {
-            fprintf(stderr, "Bad ttl : [%u]\n", (unsigned int) iphead.ip_ttl);
+            logfile(LOG_WARNING, _("Bad TTL: [%u]"),                    
+                    (unsigned int) iphead.ip_ttl);
             return;
         }        
         if (ch.carp_version != CARP_VERSION) {
-            fprintf(stderr, "Bad version : [%u]\n",
+            logfile(LOG_WARNING, _("Bad version: [%u]"),
                     (unsigned int) ch.carp_version);
             return;
         }
         if (ch.carp_vhid != vhid) {
 #ifdef DEBUG
-            fprintf(stderr, "Ignoring vhid : [%u]\n",
+            logfile(LOG_NOTICE, _("Ignoring vhid: [%u]"),
                     (unsigned int) ch.carp_vhid);
 #endif
             return;            
         }
-        if (cksum(sp, caplen) != 0) {
-            fprintf(stderr, "Bad IP checksum\n");
+        if (cksum(sp, ip_len + sizeof ch) != 0) {
+            logfile(LOG_WARNING, _("Bad IP checksum"));
             return;
         }        
         {
@@ -488,13 +504,14 @@ static void packethandler(unsigned char *dummy,
             SHA1Final(md2, &ctx);
             
             if (sizeof md2 != sizeof ch.carp_md) {
-                fprintf(stderr, "sizeof md2 != sizeof carp_md !!!\n");
+                logfile(LOG_ERR, "sizeof md2 != sizeof carp_md !!!");
                 abort();
             }
             if (memcmp(md2, ch.carp_md, sizeof md2) != 0) {
-                fprintf(stderr, "Bad digest - "
-                        "md2=[%02x%02x%02x%02x...] md=[%02x%02x%02x%02x...]\n"
-                        "Check vhid, password and virtual IP address\n",
+                logfile(LOG_WARNING,
+                        _("Bad digest - "
+                          "md2=[%02x%02x%02x%02x...] md=[%02x%02x%02x%02x...] - "
+                          "Check vhid, password and virtual IP address"),
                         (unsigned int) md2[0], (unsigned int) md2[1],
                         (unsigned int) md2[2], (unsigned int) md2[3],
                         (unsigned int) (ch.carp_md)[0],
@@ -524,9 +541,9 @@ static void packethandler(unsigned char *dummy,
             (ch.carp_advskew * 1000000ULL / 256ULL);
         
 #ifdef DEBUG
-        printf("sc_tv(local) = [%lu] ch_tv(remote) = [%lu]\n",
-               (unsigned long) sc_tv.tv_sec,
-               (unsigned long) ch_tv.tv_sec);
+        logfile(LOG_DEBUG, "sc_tv(local) = [%lu] ch_tv(remote) = [%lu]",
+                (unsigned long) sc_tv.tv_sec,
+                (unsigned long) ch_tv.tv_sec);
 #endif
         
         switch (sc.sc_state) {
@@ -537,12 +554,12 @@ static void packethandler(unsigned char *dummy,
              * If we receive an advertisement from a master who's going to
              * be more frequent than us, go into BACKUP state.
              */
-            if (timercmp(&sc_tv, &ch_tv, >) ||
-                timercmp(&sc_tv, &ch_tv, ==)) {
+            if (timercmp(&ch_tv, &sc_tv, <=)) {
                 sc.sc_ad_tmo.tv_sec = 0;
                 carp_set_state(&sc, BACKUP);
                 carp_setrun(&sc, 0);
-                puts("Going back to BACKUP state");
+                logfile(LOG_WARNING, _("Preferred master advertised: "
+                                       "going back to BACKUP state"));
             }
             break;
         case BACKUP:
@@ -552,7 +569,7 @@ static void packethandler(unsigned char *dummy,
              */
             if (preempt != 0 && timercmp(&sc_tv, &ch_tv, <)) {
                 carp_master_down(&sc);
-                puts("Putting MASTER down - preemption");                
+                logfile(LOG_WARNING, _("Putting MASTER down - preemption"));
                 break;
             }
             
@@ -564,7 +581,8 @@ static void packethandler(unsigned char *dummy,
             sc_tv.tv_sec = (unsigned int) sc.sc_advbase * dead_ratio;
             if (timercmp(&sc_tv, &ch_tv, <)) {
                 carp_master_down(&sc);
-                puts("Putting MASTER DOWN (going to time out)");
+                logfile(LOG_WARNING, 
+                        _("Putting MASTER DOWN (going to time out)"));
                 break;
             }
             
@@ -586,12 +604,31 @@ RETSIGTYPE sighandler_exit(const int sig)
 {
     (void) sig;
 #ifdef DEBUG
-    printf("sighandler_exit(): Calling [%s] and exiting\n", downscript);
+    logfile(LOG_DEBUG, "sighandler_exit(): Calling [%s] and exiting",
+            downscript);
 #endif
     (void) spawn_handler(dev_desc_fd, downscript);
     _exit(EXIT_SUCCESS);
 }
 
+char *build_bpf_rule(void)
+{
+    static char rule[256];
+    const char *srcip_s;    
+    
+    if ((srcip_s = inet_ntoa(srcip)) == NULL) {
+        logfile(LOG_ERR, "inet_ntoa: [%s]", strerror(errno));
+        return NULL;
+    }
+    snprintf(rule, sizeof rule, "proto %u and src host not %s",
+             (unsigned int) IPPROTO_CARP, srcip_s);
+#ifdef DEBUG
+    logfile(LOG_DEBUG, "BPF rule: [%s]", rule);
+#endif
+    
+    return rule;
+}
+
 int docarp(void)
 {
     struct bpf_program bpfp;
@@ -603,7 +640,6 @@ int docarp(void)
     sc.sc_advbase = advbase;
     sc.sc_advskew = advskew;
     sc.sc_init_counter = 1;
-    sc.sc_naddrs = sc.sc_naddrs6 = 0;
 #ifdef INET6
     sc.sc_im6o.im6o_multicast_hlim = CARP_DFLTTL;
 #endif /* INET6 */
@@ -612,7 +648,7 @@ int docarp(void)
         const size_t passlen = strlen(pass) + (size_t) 1U;
         
         if (passlen > sizeof sc.sc_key) {
-            fprintf(stderr, "Password too long\n");
+            logfile(LOG_ERR, _("Password too long"));
             return -1;
         }
         memcpy(sc.sc_key, pass, passlen);
@@ -625,22 +661,25 @@ int docarp(void)
     carp_hmac_prepare(&sc);
 
     if (fill_mac_address() != 0) {
-        fprintf(stderr, "Unable to manage interface [%s]\n", interface);
+        logfile(LOG_ERR, _("Unable to find MAC address of [%s]"),
+                interface);
         return -1;
     }
-    printf("Local advertised ethernet address is "
-           "[%02x:%02x:%02x:%02x:%02x:%02x]\n",
-           (unsigned int) hwaddr[0], (unsigned int) hwaddr[1],
-           (unsigned int) hwaddr[2], (unsigned int) hwaddr[3],
-           (unsigned int) hwaddr[4], (unsigned int) hwaddr[5]);
-    if ((dev_desc = pcap_open_live(interface, ETHERNET_MTU, 1,
+    logfile(LOG_INFO, _("Local advertised ethernet address is "
+                        "[%02x:%02x:%02x:%02x:%02x:%02x]"),
+            (unsigned int) hwaddr[0], (unsigned int) hwaddr[1],
+            (unsigned int) hwaddr[2], (unsigned int) hwaddr[3],
+            (unsigned int) hwaddr[4], (unsigned int) hwaddr[5]);
+    if ((dev_desc = pcap_open_live(interface, ETHERNET_MTU, 0,
                                    CAPTURE_TIMEOUT, errbuf)) == NULL) {
-        fprintf(stderr, "pcap_open_live() : %s [%s]\n", errbuf, interface);
+        logfile(LOG_ERR, _("Unable to open interface [%s]: %s"),
+                interface, errbuf);
         return -1;
     }    
-    if (pcap_compile(dev_desc, &bpfp, (char *) BPF_CARP_RULE,
+    if (pcap_compile(dev_desc, &bpfp, build_bpf_rule(),
                      1, (bpf_u_int32) 0) != 0) {
-        fprintf(stderr, "pcap_compile() : %s [%s]\n", errbuf, interface);
+        logfile(LOG_ERR, _("Unable to compile pcap rule: %s [%s]"),
+                errbuf, interface);
         return -1;
     }
     pcap_setfilter(dev_desc, &bpfp);
@@ -657,16 +696,17 @@ int docarp(void)
     }
     
     for (;;) {
-        nfds = poll(pfds, (nfds_t) 1, (int) (sc.sc_advbase * 1000));
+        nfds = poll(pfds, (nfds_t) 1, sc.sc_advbase * 1000);
         if (nfds == -1 || 
             (pfds[0].revents & (POLLERR | POLLHUP | POLLNVAL)) != 0) {
 #ifdef DEBUG
-            perror("poll error");
+            logfile(LOG_NOTICE, "poll() error");
 #endif
             break;
         }
         if (gettimeofday(&now, NULL) != 0) {
-            perror("gettimeofday()");
+            logfile(LOG_WARNING, "gettimeofday() failed: %s",
+                    strerror(errno));
             continue;
         }        
         if (nfds == 1) {
@@ -675,9 +715,11 @@ int docarp(void)
         if (sc.sc_md_tmo.tv_sec != 0 && timercmp(&now, &sc.sc_md_tmo, >)) {
             carp_master_down(&sc);
         }
+#ifdef INET6
         if (sc.sc_md6_tmo.tv_sec != 0 && timercmp(&now, &sc.sc_md6_tmo, >)) {
             carp_master_down(&sc);
         }
+#endif
         if (sc.sc_ad_tmo.tv_sec != 0 && timercmp(&now, &sc.sc_ad_tmo, >)) {
             carp_send_ad(&sc);
         }
