@@ -22,5 +22,8 @@ GLOBAL0(char *downscript);
 GLOBAL0(signed char preempt);
 GLOBAL0(signed char shutdown_at_exit);
 GLOBAL0(unsigned char hwaddr[6]);
+GLOBAL0(signed char no_syslog);
+GLOBAL0(signed char daemonize);
+GLOBAL(int syslog_facility, DEFAULT_FACILITY);
 
 #endif
