@@ -1,7 +1,7 @@
 #ifndef __CARP_P_H__
 #define __CARP_P_H__ 1
 
-static const char *GETOPT_OPTIONS = "i:s:v:p:Pa:hb:k:u:d:r:zB";
+static const char *GETOPT_OPTIONS = "i:s:v:p:Pa:hb:k:u:d:r:zf:B";
 
 static struct option long_options[] = {
     { "interface", 1, NULL, 'i' },
@@ -17,6 +17,7 @@ static struct option long_options[] = {
     { "downscript", 1, NULL, 'd' },
     { "deadratio", 1, NULL, 'r' },
     { "shutdown", 0, NULL, 'z' },
+    { "facility", 1, NULL, 'f' },
     { "daemonize", 0, NULL, 'B' },
     { NULL, 0, NULL, 0 }
 };
